//  Copyright (C) 2002-2003 Intel Corporation, All Rights Reserved.
//  Permission is hereby granted to merge this program code with 
//  other program material to create a derivative work.  This 
//  derivative work may be distributed in compiled object form only.
//  Any other publication of this program, in any form, without the 
//  explicit permission of the copyright holder is prohibited.
//
//  Send questions and comments to erik.j.johnson@intel.com, 
//  aaron.kunze@intel.com

//-------------------------------------------------------------------
// count.uc - Chapter 5
//

#ifndef COUNT_UC
#define COUNT_UC

#include "stdmac.uc"
#include "bpf.h"

//-------------------------------------------------------------------
// count_init
//
//    Description:
//       Initialize the packet and byte counters.
//
//    Parameters:
//      Outputs: n/a
//      In/Outs: n/a
//      Inputs: n/a
//      Constants: n/a
//      Labels: n/a
//
//    Side effects: n/a
//
//    See also: n/a
//
#macro count_init()
.begin
	.if (ctx() == 0)
		.reg addr
		.reg $zero_xfer
		.sig counter_sig

		immed32($zero_xfer, 0)

		// Initialize the counters
		immed[addr,DROP_COUNT_ADDR_0]
		scratch[write, $zero_xfer, addr, 0, 1],
				ctx_swap[counter_sig]
		immed[addr,RX_PKT_COUNT_ADDR_0]
		scratch[write, $zero_xfer, addr, 0, 1],
				ctx_swap[counter_sig]
		immed[addr,RX_BYTE_COUNT_ADDR_0]
		scratch[write, $zero_xfer, addr, 0, 1],
				ctx_swap[counter_sig]
	.endif
.end
#endm


//-------------------------------------------------------------------
// count
//
//    Description:
//       Update the packet and byte counters.
//
//    Parameters:
//      Outputs: n/a
//      In/Outs: n/a
//      Inputs: n/a
//      Constants: n/a
//      Labels: n/a
//
//    Side effects: n/a
//
//    See also: n/a
//
#macro count()
.begin
	.reg $buf_length_xfer buf_length
	.reg addr
	.sig counter_sig
	
	immed[addr, RX_PKT_COUNT_ADDR_0]
	scratch[incr, --, addr, 0]

	dl_meta_get_buffer_size(buf_length)
	move($buf_length_xfer, buf_length)
	immed[addr,RX_BYTE_COUNT_ADDR_0]
	scratch[add, $buf_length_xfer, addr, 0],
		ctx_swap[counter_sig]

	immed32(dl_next_block, COUNT_NEXT_BLOCK)
.end
#endm

#endif // COUNT_UC