/******************************************************************************
 *
 *	mthood_linux_app.c
 *	--------------------
 *	Linux example applicaton for mthood
 *
 ******************************************************************************
 *
 *	COPYRIGHT 2003 BY RADISYS CORPORATION.  ALL RIGHTS RESERVED.
 *
 ******************************************************************************/

#include <stdint.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <enpv2_types.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <signal.h>

/******************************************************************************
 *	BUILD SWITCHES
 ******************************************************************************/
/* loading spi3br.o will do this for us */
#define SLOW_PORT_FIX					FALSE


/******************************************************************************
 *	FORWARD DECLARATIONS
 ******************************************************************************/
extern void TestSPI3Interrupts(void);
extern void StartMicroEngines(void);
extern void StopMicroEngines(void);
extern void PrintCount();
extern void ZeroCount();

/******************************************************************************
 *
 *							APPLICATION FUNCTIONS
 *
 ******************************************************************************/
void ApplicationStart( void )
{

#if( SLOW_PORT_FIX == TRUE )
	/* put slowport in corect mode if not already */
	spfix();
#endif

	/* init and start spi3br */
	StartSpi3br();


	/* init and start macs */
	StartMacs();

	/* Zero out the counters */
	ZeroCount();

	/* Initialize microengines */
	StartMicroEngines();
}

void ApplicationStop( void )
{
  StopMicroEngines();
  StopMacs();
  StopSpi3br();
}

int main(int argc, char **argv)
{
	if (argc != 2) {
		printf("Usage: BPFInit [ start | stop | showmacstats | ");
		printf("showmacs | testints | spi3br ]\n");
		return 0;
	}

	if (strncmp(argv[1], "start", 5) == 0) {
	  ApplicationStart();
	} else if (strncmp(argv[1], "stop", 4) == 0) {
	  ApplicationStop();
	} else if (strncmp(argv[1], "show", 4) == 0) {
	  PrintCount();
	} else if (strncmp(argv[1], "tstart", 6) == 0) {
	  while( 1 ) {
	    sleep( 30 );
	    PrintCount();
	  }
	  /*
	    struct itimerval time = {{30L,0L},{1L,0L}};
	    if( signal( SIGALRM, DCWrapper) == SIG_ERR ) {
	    perror( "Register signal handler failed" );
	    exit( EXIT_FAILURE );
	    }
	    if( setitimer( ITIMER_REAL, &time, NULL ) == -1 ) {
	    perror( "Setitimer() failed" );
	    exit( EXIT_FAILURE );
	    }
	  */
	} else if (strncmp(argv[1], "showmacstats", 12) == 0) {
		ShowMacStats();
	} else if (strncmp(argv[1], "showmacs", 8) == 0) {
		printf("\n------ MAC 0 ------\n");
		ShowMac(0);
		printf("\n\n------ MAC 1 ------\n");
		ShowMac(1);
        } else if (strncmp(argv[1], "spi3bridge", 10) == 0) {
		ShowSpi3br();
    } else if (strncmp(argv[1], "testints", 8) == 0) {
		TestSPI3Interrupts();
	} else {
		printf("Invalid parameter\n");
	}
	return 0;
}



