#define SCRATCH_RWS_OFFSET 0xCA000000
#define SCRATCH_SUB_OFFSET 0xCB000000

#define MAX_COUNTER 128

#define XLAT_ERROR 0xFFFFFFFF

#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>

#include <uclo.h>
#include <halMev2Api.h>

void ZeroCount( void ) {
  uint32_t *pcount;
  int i;

  /* Initialize HAL lib */
  if( halMe_Init( 0 ) != HALME_SUCCESS ) {
    printf( "Failed to initialize HAL library\n" );
    exit( EXIT_FAILURE );
  }

  /* Get pointer to counters */
  if(( pcount = (uint32_t *)
       halMe_GetVirAddr( SCRATCH_RWS_OFFSET )) == (uint32_t *) XLAT_ERROR ) {
    printf( "Error getting counter virtual address\n" );
    exit( EXIT_FAILURE );
  }

  for( i = 0; i < MAX_COUNTER; ++i ) {
    *(pcount++) = 0;
  }
}

void PrintCount( void ) {
  uint32_t *pcount;
  uint32_t counts[6];
  int i;

  /* Initialize HAL lib */
  if( halMe_Init( 0 ) != HALME_SUCCESS ) {
    printf( "Failed to initialize HAL library\n" );
    exit( EXIT_FAILURE );
  }

  /* Get pointer to counters for reading */
  if(( pcount = (uint32_t *)
       halMe_GetVirAddr( SCRATCH_RWS_OFFSET )) == (uint32_t *) XLAT_ERROR ) {
    printf( "Error getting counter virtual address: read\n" );
    exit( EXIT_FAILURE );
  }

  for( i = 0; i < 6; ++i ) {
    counts[i] = *(pcount++);
  }

  /* Get pointer to counters for atomic subtract */
  if(( pcount = (uint32_t *)
       halMe_GetVirAddr( SCRATCH_SUB_OFFSET )) == (uint32_t *) XLAT_ERROR ) {
    printf( "Error getting counter virtual address: subtract\n" );
    exit( EXIT_FAILURE );
  }

  for( i = 0; i < 6; ++i ) {
    *(pcount++) = counts[i];
  }

  /* Finally print the counts to screen */
  for( i = 0; i < 2; ++i ) {
    printf( "Drop Packet Count %d = %u\n", i, counts[i*3+0] );
    printf( "Enqueue Count %d = %u\n", i, counts[i*3+1] );
    printf( "Enqueue Byte Count %d = %u\n", i, counts[i*3+2] );
  }
}
  
