#include <stdlib.h>
#include <uclo.h>
#include <halMev2Api.h>

#define UCODE_FILE "/opt/enp2611/rx_count.uof"

/* NOTE: for Ueng mask least significant 16 bits represent cluster 0
   and most significant 16 bits represent cluster 1 */

void StartMicroEngines() {
  void *handle;
  unsigned meMask;
  int i;
  
  /* Initialize ucode library */
  UcLo_InitLib();
  
  /* Initialize microengines, assume 4 MEs per cluster */
  UcLo_InitLibUeng( 0x000F000F );
  
  /* Read ucode from file to memory */
  if( UcLo_LoadObjFile((void **) &handle, UCODE_FILE ) != UCLO_SUCCESS) {
    printf( "Unable to load ucode from file\n" );
    exit( EXIT_FAILURE );
  }
  
  printf( "ME Mask = %x\n", (meMask = UcLo_GetAssignedMEs((void *) handle )));
  
  if( UcLo_WriteUimageAll((void *) handle ) != UCLO_SUCCESS ) {
    printf( "Unable to load ucode to MEs\n" );
    exit( EXIT_FAILURE );
  }

  /* Start MEs */
  for( i = 0; i < sizeof(meMask)*8; ++i ) {
    if((( meMask >> i ) & 1) == 1 ) {
      if( halMe_Start( i, 0xff /* All contexts */) != HALME_SUCCESS ) {
	printf( "Failed to start ME %d\n", i );
	exit( EXIT_FAILURE );
      } else {
	printf( "Successfully started ME %d\n", i );
      }
    }
  }
}
  

void StopMicroEngines() {
  /* Put all MEs and all contexts into reset */
  halMe_Reset( 0x000F000F, 1 );
  printf( "Microengines stopped\n" );
}
